import java.io.FileOutputStream;
import java.io.PrintStream;
import java.io.IOException;

public class ExactReoptimization_OnlineOptimization {

	public static void main(String[] args) throws IOException {
		int numberOfInstances = 50;
		int instanceLength = 25;
		int[] lookaheads = new int[]{1, 2, 5, 10}; // "1" ist purer Online-Fall, d.h. von "1" nach "2", "5", ... erkennt man den prozessualen und informationalen Lookahead-Effekt
		int[] maxGradualHorizons = new int[]{5, 10, 15, 9999}; // "9999" entspricht Forecasting bis ans Inputsequenz-Ende
		double[] gradualLossFactors_BinPacking = new double[]{0, 0.05, 0.1, 0.2}; // "0" entspricht deterministischem Lookahead-Forecasting
		double[] gradualLossFactors_TSP = new double[]{0, 0.05, 0.1, 0.2};
		double[] gradualLossFactors_LotSizing = new double[]{0, 5, 10, 20};
		double[] gradualLossFactors_Scheduling = new double[]{0, 2.5, 5, 10};		
		boolean binPacking = false; boolean tsp = false; boolean lotSizing = false; boolean scheduling = true;
		//Bin Packing
		if(binPacking){
			Instance_BinPacking[] instances_BinPacking = new Instance_BinPacking[numberOfInstances];
			for(int i = 0; i < numberOfInstances; i++)
				instances_BinPacking[i] = new Instance_BinPacking(i, instanceLength);
			PrintStream outputBinPacking = new PrintStream(new FileOutputStream("outputBinPacking.txt"));		
			for(int a = 0; a < lookaheads.length; a++)
				for(int b = 0; b < maxGradualHorizons.length; b++)
					for(int c = 0; c < gradualLossFactors_BinPacking.length; c++)			
						for(int i = 0; i < numberOfInstances; i++){
							outputBinPacking.print(lookaheads[a] + "\t" + maxGradualHorizons[b] + "\t" + gradualLossFactors_BinPacking[c] + "\t" + i + "\t");
							outputBinPacking.print(Algorithms_BinPacking.solveInstance("BFD", gradualLossFactors_BinPacking[c], maxGradualHorizons[b], instances_BinPacking[i], lookaheads[a]) + "\t");
							outputBinPacking.print(Algorithms_BinPacking.solveInstance("SnapshotMIP", gradualLossFactors_BinPacking[c], maxGradualHorizons[b], instances_BinPacking[i], lookaheads[a]) + "\t");
							outputBinPacking.println(Algorithms_BinPacking.solveInstance("SnapshotMIPForecasts", gradualLossFactors_BinPacking[c], maxGradualHorizons[b], instances_BinPacking[i], lookaheads[a]));
						}
			outputBinPacking.close();
		}
		//TSP
		if(tsp){
			Instance_TSP[] instances_TSP = new Instance_TSP[numberOfInstances];
			for(int i = 0; i < numberOfInstances; i++)
				instances_TSP[i] = new Instance_TSP(i, instanceLength);
			PrintStream outputTSP = new PrintStream(new FileOutputStream("outputTSP.txt"));
			for(int a = 0; a < lookaheads.length; a++)
				for(int b = 0; b < maxGradualHorizons.length; b++)
					for(int c = 0; c < gradualLossFactors_TSP.length; c++)	
						for(int i = 0; i < numberOfInstances; i++){
							outputTSP.print(lookaheads[a] + "\t" + maxGradualHorizons[b] + "\t" + gradualLossFactors_TSP[c] + "\t" + i + "\t");
							outputTSP.print(Algorithms_TSP.solveInstance("NN", gradualLossFactors_TSP[c], maxGradualHorizons[b], instances_TSP[i], lookaheads[a]) + "\t");
							outputTSP.print(Algorithms_TSP.solveInstance("SnapshotMIP", gradualLossFactors_TSP[c], maxGradualHorizons[b], instances_TSP[i], lookaheads[a]) + "\t");
							outputTSP.println(Algorithms_TSP.solveInstance("SnapshotMIPForecasts", gradualLossFactors_TSP[c], maxGradualHorizons[b], instances_TSP[i], lookaheads[a]) + "\t");
						}
			outputTSP.close();
		}
		//LotSizing
		if(lotSizing){
			Instance_LotSizing[] instances_LotSizing = new Instance_LotSizing[numberOfInstances];
			for(int i = 0; i < numberOfInstances; i++)
				instances_LotSizing[i] = new Instance_LotSizing(i, instanceLength);
			PrintStream outputLotSizing = new PrintStream(new FileOutputStream("outputLotSizing.txt"));
			for(int a = 0; a < lookaheads.length; a++)
				for(int b = 0; b < maxGradualHorizons.length; b++)
					for(int c = 0; c < gradualLossFactors_LotSizing.length; c++)	
						for(int i = 0; i < numberOfInstances; i++){
							outputLotSizing.print(lookaheads[a] + "\t" + maxGradualHorizons[b] + "\t" + gradualLossFactors_LotSizing[c] + "\t" + i + "\t");
							outputLotSizing.print(Algorithms_LotSizing.solveInstance("SM", gradualLossFactors_LotSizing[c], maxGradualHorizons[b], instances_LotSizing[i], lookaheads[a]) + "\t");
							outputLotSizing.print(Algorithms_LotSizing.solveInstance("SnapshotMIP", gradualLossFactors_LotSizing[c], maxGradualHorizons[b], instances_LotSizing[i], lookaheads[a]) + "\t");
							outputLotSizing.println(Algorithms_LotSizing.solveInstance("SnapshotMIPForecasts", gradualLossFactors_LotSizing[c], maxGradualHorizons[b], instances_LotSizing[i], lookaheads[a]) + "\t");
						}
			outputLotSizing.close();
		}
		//Scheduling
		if(scheduling){
			Instance_Scheduling[] instances_Scheduling = new Instance_Scheduling[numberOfInstances];
			for(int i = 0; i < numberOfInstances; i++)
				instances_Scheduling[i] = new Instance_Scheduling(i, instanceLength);
			PrintStream outputScheduling = new PrintStream(new FileOutputStream("outputScheduling.txt"));
			for(int a = 0; a < lookaheads.length; a++)
				for(int b = 0; b < maxGradualHorizons.length; b++)
					for(int c = 0; c < gradualLossFactors_Scheduling.length; c++)	
						for(int i = 0; i < numberOfInstances; i++){
							outputScheduling.print(lookaheads[a] + "\t" + maxGradualHorizons[b] + "\t" + gradualLossFactors_Scheduling[c] + "\t" + i + "\t");
							outputScheduling.print(Algorithms_Scheduling.solveInstance("Heuristic", gradualLossFactors_Scheduling[c], maxGradualHorizons[b], instances_Scheduling[i], lookaheads[a]) + "\t");
							outputScheduling.print(Algorithms_Scheduling.solveInstance("SnapshotMIP", gradualLossFactors_Scheduling[c], maxGradualHorizons[b], instances_Scheduling[i], lookaheads[a]) + "\t");
							outputScheduling.println(Algorithms_Scheduling.solveInstance("SnapshotMIPForecasts", gradualLossFactors_Scheduling[c], maxGradualHorizons[b], instances_Scheduling[i], lookaheads[a]) + "\t");
						}
			outputScheduling.close();
		}
	}
}